#!/usr/bin/env python
# coding=utf-8
# Filename: chatbot.py
# Author: Tamas Gal <tgal@km3net.de>
"""
The monitoring chatbot.

Usage:
    chatbot.py
    chatbot.py (-h | --help)

Options:
    -h --help   Show this screen.

"""
import re
import requests
import subprocess
import time
import toml
from rocketchat_API.rocketchat import RocketChat
from RocketChatBot import RocketChatBot

import km3pipe as kp

log = kp.logger.get_logger("chatbot")

URL = "https://chat.km3net.de"
CONFIG = "pipeline.toml"
RECONNECT_INTERVAL = 30

with open(CONFIG, 'r') as fobj:
    print(f"Reading configuration from {CONFIG}")
    config = toml.load(fobj)
    BOTNAME = config['Alerts']['botname']
    PASSWORD = config['Alerts']['password']
    CHANNEL = config['Alerts']['channel']


def get_channel_id(channel):
    print(f"Getting channel ID for channel: {channel}")
    tries = 0
    while True:
        tries += 1

        try:
            rocket = RocketChat(BOTNAME, PASSWORD, server_url=URL)
        except requests.exceptions.ConnectionError as e:
            log.error(f"Unable to connect to the RocketChat server: {e}")
        except Exception as e:
            log.error(f"Unknown error occured: {e}")
        else:
            break

        interval = tries * RECONNECT_INTERVAL
        print(f"Reconnecting in {interval} seconds...")
        time.sleep(interval)

    channels = rocket.channels_list(count=0).json()['channels']
    for c in channels:
        print(f"   -> {c['name']} => {c['_id']}")
        if c['name'] == channel:
            print(f"Found channel ID for {channel} is {c['_id']}")
            return c['_id']
    log.error("No channel found with name {}".format(channel))


CHANNEL_ID = get_channel_id(CHANNEL)


def run():
    print("Running the monitoring bot system")
    bot = spawn_bot()
    register_handlers(bot)
    bot.send_message("I am up and running!", CHANNEL_ID)
    bot.run()


def spawn_bot():
    print("Spawning the bot")
    return RocketChatBot(BOTNAME, PASSWORD, URL)


def is_shifter(user):
    print(f"Checking if {user} is a shifter")
    with open(CONFIG, 'r') as fobj:
        config = toml.load(fobj)
        try:
            alerts_config = config['Alerts']
        except KeyError:
            log.error("No 'Alerts' section found in the configuration file")
            return False
        try:
            return user in alerts_config['shifters']
        except KeyError:
            log.error("No 'shifters' section found in 'Alerts' of the configuration file")
            return False


def is_operator(user):
    print(f"Checking if {user} is an operator")
    with open(CONFIG, 'r') as fobj:
        config = toml.load(fobj)
        return user in config['Alerts']['operators']


def register_handlers(bot):
    print("Registering API handlers")
    def greet(msg, user, channel_id):
        if channel_id != CHANNEL_ID:
            print("skipping")
            return
        bot.send_message('hello @' + user, channel_id)

    def status(msg, user, channel_id):
        print(f"Reporting status to channel {channel_id}")
        if channel_id != CHANNEL_ID:
            print(f"Skipping channel with ID {channel_id}")
            return
        if not is_shifter(user) and not is_operator(user):
            bot.send_message(
                "Sorry @{}, only operators and shifters are allowed to mess "
                "with me, sorry...".format(user), channel_id)
            return
        print("Asking subservisorctl for the status")
        try:
            status = "```\n" + subprocess.check_output(
                ['supervisorctl', 'status']).decode('ascii') + "\n```"
        except subprocess.CalledProcessError as e:
            status = "```\n{}\n```".format(e.output.decode('ascii'))
        print("Sending status")
        bot.send_message(status, channel_id)

    def supervisorctl(msg, user, channel_id):
        if channel_id != CHANNEL_ID:
            print("skipping")
            return
        if not is_shifter(user) and not is_operator(user):
            bot.send_message(
                "Sorry @{}, only operators and shifters are allowed to mess "
                "with me, sorry...".format(user), channel_id)
            return
        try:
            output = subprocess.check_output(
                ['supervisorctl'] + msg.split(),
                stderr=subprocess.STDOUT).decode('ascii')
        except subprocess.CalledProcessError as e:
            output = e.output.decode('ascii')
        print("supervisorctl ({}), called by {}:\n{}".format(
            msg, user, output))
        bot.send_message(output, channel_id)

    def shifters(msg, user, channel_id):
        if channel_id != CHANNEL_ID:
            print("skipping")
            return
        if not is_operator(user):
            bot.send_message(
                "Sorry @{}, only operators are allowed to set shifters!".
                format(user), channel_id)
            return
        try:
            with open(CONFIG, 'r') as fobj:
                config = toml.load(fobj)
            shifters = msg[3:].strip()
            config['Alerts']['shifters'] = shifters
            with open(CONFIG, 'w') as fobj:
                toml.dump(config, fobj)
            msg = f'Alright, the new shifters are {shifters}, welcome!'
            print(msg)
            bot.send_message(msg, channel_id)
        except Exception as e:
            bot.send_message(f'something went horribly wrong... {e}',
                             channel_id)

    def help(msg, user, channel_id):
        if channel_id != CHANNEL_ID:
            print("skipping", channel_id)
            return
        help_str = f"""
        Hi @{user} I was built to take care of the monitoring alerts.
        Here is how you can use me:
        - `@{BOTNAME} shifters are cnorris and bspencer`
          -> set the new shifters who I may annoy with chat messages and
          emails.
        - `@{BOTNAME} status` -> show the status of the monitoring system
        - `@{BOTNAME} supervisorctl` -> take control over the monitoring system
        - `@{BOTNAME} help` -> show this message
        """

        bot.send_message(help_str, channel_id)

    handlers = [(['hello', 'hey', 'hi', 'ciao'], greet), (['status'], status),
                (['help'], help), (['shifters'], shifters),
                (['supervisorctl'], supervisorctl)]
    for trigger, handler in handlers:
        bot.add_dm_handler(trigger, handler)
    print("All handlers are registered")


def main():
    from docopt import docopt
    args = docopt(__doc__)

    run()


if __name__ == '__main__':
    main()
