#!/usr/bin/env python
# coding=utf-8
# Filename: log_analyser.py
# Author: Tamas Gal <tgal@km3net.de>
# vim: ts=4 sw=4 et
from collections import defaultdict
import gzip
import sys
import re
import numpy as np
import matplotlib

# Force matplotlib to not use any Xwindows backend.
matplotlib.use("Agg")
import gc
import matplotlib.pyplot as plt
import os
import datetime
from datetime import datetime as dt
from datetime import timezone as tz
import time

# Event names and colours
EVENTS = dict(ERROR="red", WARNING="orange", Died="deeppink", Born="steelblue")
BUFFER_SIZE = 16 * 1024**2  # buffer size for the lines when parsing the log
REGEX_LOG_LINE = re.compile(".+ ([A-Za-z]+) \[([A-Za-z]+)\]: .+")
REGEX_LEGACY_LOG_LINE = re.compile("^.+ \[([A-Za-z]+)\]: .+")


def plot_log_statistics(out_file, summary, title):
    """Creates a categorical bar plot for each event and process"""
    processes = sorted(summary.keys())
    xs = np.arange(len(processes))
    w = 0.8 / len(EVENTS)

    fig, ax = plt.subplots()
    for idx, (event, color) in enumerate(EVENTS.items()):
        x_offset = idx * w + w / 2 - w * len(EVENTS) / 2
        ax.bar(
            xs + x_offset,
            [summary[process][event] for process in processes],
            width=w,
            color=color,
            label=event,
        )
    ax.set_xticks(xs, processes)
    ax.set_ylabel("count")
    ax.legend()
    ax.set_ylim(1e-1, 1e6)
    ax.set_yscale("log")
    ax.grid(True)
    ax.set_title(title)

    plt.savefig(out_file)
    plt.close("all")
    gc.collect()


def seconds_to_UTC_midnight():
    """Returns the seconds until next midnight"""
    tomorrow = dt.now(tz.utc) + datetime.timedelta(days=1)
    midnight = dt(
        year=tomorrow.year,
        month=tomorrow.month,
        day=tomorrow.day,
        hour=0,
        minute=0,
        second=0,
        tzinfo=tz.utc,
    )
    return (midnight - dt.now(tz.utc)).seconds


def process_log_file(log_file):
    """Generates a dictionary of event counts in a log file

    The returned dictionary has the structure dict[PROCESS][EVENT] => count.
    """
    summary = defaultdict(lambda: defaultdict(int))

    if log_file.endswith(".gz"):
        opener = gzip.open
        filemode = "rt"
    else:
        opener = open
        filemode = "r"

    n_lines_parsed = 0
    n_lines_unparsed = 0
    with opener(log_file, filemode) as fobj:
        lines_chunk = fobj.readlines(BUFFER_SIZE)
        while lines_chunk:
            for line in lines_chunk:
                m = REGEX_LOG_LINE.match(line)
                if m is not None:
                    tag = m[1]
                    process = m[2]
                else:
                    m = REGEX_LEGACY_LOG_LINE.match(line)
                    if m is not None:
                        tag = "MSG"
                        process = m[1]
                    else:
                        n_lines_unparsed += 1
                        continue
                if tag in ("Born", "Died"):
                    summary[process][tag] += 1
                for severity in ("WARNING", "ERROR"):
                    if (
                        severity in line
                        or severity.lower() in line
                        or severity.lower().capitalize() in line
                    ):
                        summary[process][severity] += 1
                n_lines_parsed += 1
            lines_chunk = fobj.readlines(BUFFER_SIZE)

    print(f"  parsed lines: {n_lines_parsed}")
    print(f"  unparsed lines: {n_lines_unparsed}")
    for process, stats in summary.items():
        print(f"    {process}:")
        for event, n_lines in stats.items():
            print(f"      {event}: {n_lines}")

    return summary


def main():

    log_dir = "/logs/"
    regexp = "^MSG_(.+)\.log"

    while True:
        for fname in os.listdir(log_dir):
            plot_fpath = os.path.join(log_dir, os.path.splitext(fname.replace(".gz", ""))[0] + ".png")
            log_fpath = os.path.join(log_dir, fname)
            if re.match(regexp, fname) and not os.path.exists(plot_fpath):
                print("-> Processing ", fname)
                summary = process_log_file(log_fpath)
                title = os.path.basename(fname)
                plot_log_statistics(plot_fpath, summary, title)
        time.sleep(seconds_to_UTC_midnight() + 5 * 60)


if __name__ == "__main__":
    main()
