#!/usr/bin/envOA pytAhonAAA
# coding=utf-8
# Filename: dom_rates.py
# Author: Tamas Gal <tgal@km3net.de>, Alba Domi <adomi@km3net.de>
# vim: ts=4 sw=4 et
"""
Monitors triggered hits.

Usage:
    triggered_hits.py [options]
    triggered_hits.py (-h | --help)

Options:
    -l LIGIER_IP    The IP of the ligier [default: 127.0.0.1].
    -p LIGIER_PORT  The port of the ligier [default: 5553].
    -d DET_ID       Detector ID [default: 29].
    -o PLOT_DIR     The directory to save the plot [default: /plots].
    -h --help       Show this screen.

"""
from __future__ import division

from io import BytesIO
import os

import numpy as np
import numba as nb
import collections as cs
import gc
import itertools
from datetime import datetime

from collections import Counter
import multiprocessing as mp
import time

import matplotlib

matplotlib.use("Agg")
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
import matplotlib.colors as colors
from matplotlib import pylab  # noqa

import km3pipe as kp
import km3pipe.style
import km3io
from km3pipe.io.daq import DAQPreamble, DAQEvent
from km3modules.common import MemoryObserver

VERSION = "1.0"
km3pipe.style.use("km3pipe")


class EventExtractor(kp.Module):
    def process(self, blob):
        data = blob["CHData"]
        data_io = BytesIO(data)
        preamble = DAQPreamble(file_obj=data_io)
        event = DAQEvent(file_obj=data_io)
        blob["Event"] = event
        return blob


class TriggeredHits(kp.Module):

    """Creates a coloured square for each DOM,
    representing the triggered hits, for each trigger."""

    def configure(self):
        self.plots_path = self.require("plots_path")
        det_id = self.require("det_id")
        self.calib = None
        self.run = None
        self.detector = kp.hardware.Detector(det_id=det_id)
        self.index = 0
        self.triggers = None
        self.dt = 180  # refresh time in seconds
        self.start_time = datetime.now()
        self.create_plots = kp.time.Cuckoo(self.dt, self.make_plot)
        self.n_slices = 0
        self.d = cs.defaultdict(list)
        self.log.warning("Starting triggered hits monitor")
        self.data = cs.defaultdict(int)
        self.triggers = [
            ("JTRIGGER3DMUON", km3io.tools.is_3dmuon),
            ("JTRIGGER3DSHOWER", km3io.tools.is_3dshower),
            ("JTRIGGERMXSHOWER", km3io.tools.is_mxshower),
        ]

    def process(self, blob):

        """Analyse the trigger flags for an incoming event"""

        if not str(blob["CHPrefix"].tag) == "IO_EVT":
            return blob

        run = blob["Event"].header.run
        if run != self.run:
            self.cprint(f"Updating calibration for run {run}")
            self.run = run
            self._update_calib(det_id=blob["Event"].header.det_id, run=run)

        hits = blob["Event"].triggered_hits
        calib_hits = self.calib.apply(kp.Table(hits))

        for hit in calib_hits:
            tm = int(hit.trigger_mask)
            du = hit.du
            floor = hit.floor
            for idx, (trigger_name, trigger_check) in enumerate(self.triggers):
                self.data[idx, du, floor] += int(trigger_check(tm))

        self.create_plots.msg()

        return blob

    def plot_triggered_hits(
        self,
        data,
        detector,
        dus,
        triggers,
        filename,
        vmin=0.01,
        vmax=1.0,
        cmap="winter",
        missing="lightgray",
        label="Number of Triggered Hits",
    ):

        """
        Creates a plot in the classical monitoring.km3net.de style.
        Parameters
        ----------
        data: dict((du, floor) -> (JTrigger3DMuon_hits, JTrigger3DShower_hits, JTriggerMXShower_hits))
        detector: km3pipe.hardware.Detector() instance
        filename: filename or filepath
        label: str
        title: str
        """
        x, y, _ = zip(*detector.doms.values())

        m_size = 260
        scatter_args = {
            "edgecolors": "black",
        }

        xa, ya = map(np.array, zip(*data.keys()))

        m = dict(zip(dus, np.arange(len(dus))))

        n_triggers = len(list(data.values())[0])

        maxz = max(max(data.values()))
        if maxz == 0:
            maxz = 0.1

        for n in range(0, n_triggers):
            zs = np.array([i[n] for i in list(data.values())])

            fig, ax = plt.subplots(figsize=(10, 7))

            sc_inactive = ax.scatter(
                np.vectorize(m.get)(x),
                y,
                c=missing,
                label="missing",
                s=m_size * 0.9,
                **scatter_args,
                marker="s",
            )

            # I would keep this commented for now, maybe in ARCA we will use it
            # vmax = zs.max()
            # if vmax == 0:
            #   vmax = 0.01

            sc = ax.scatter(
                np.vectorize(m.get)(xa),
                ya,
                c=zs,
                label=None,
                cmap=cmap,
                s=m_size,
                marker="s",
                norm=colors.LogNorm(vmin=vmin, vmax=maxz),
                **scatter_args,
            )

            cbar = plt.colorbar(sc, extend="both", pad=0.005)
            cbar.set_label(label, fontsize=15, labelpad=10)

            ax.set_title(
                "{0}\n{1} UTC".format(triggers[n], datetime.utcnow().strftime("%c")),
                fontsize=20,
            )
            ax.set_xlabel("STRING")
            ax.set_ylabel("FLOOR")
            ax.set_ylim(-2)
            ax.set_yticks(range(1, 18 + 1))
            ax.set_xticks(list(m.values()))
            ax.set_xticklabels(list(m.keys()))

            ax.legend(
                bbox_to_anchor=(0.0, -0.16, 0.12, 0.102),
                loc=1,
                ncol=2,
                mode="expand",
                borderaxespad=0.0,
            )

            fig.tight_layout()
            plt.savefig(filename + triggers[n] + ".png", dpi=120, bbox_inches="tight")

        plt.close("all")
        gc.collect()

    def make_plot(self):
        self.cprint(self.__class__.__name__ + ": updating plot.")

        time = datetime.now()
        dt = (time - self.start_time).total_seconds()

        d = cs.defaultdict(list)

        for k, v in self.data.items():
            if k[0] == 0:
                d[k[1], k[2]].append(v / dt)

        for k, v in self.data.items():
            if k[0] == 1:
                d[k[1], k[2]].append(v / dt)

        for k, v in self.data.items():
            if k[0] == 2:
                d[k[1], k[2]].append(v / dt)

        triggers = (self.triggers[0][0], self.triggers[1][0], self.triggers[2][0])

        filename = os.path.join(self.plots_path, "triggered_hits_")

        self.plot_triggered_hits(
            d,
            self.calib.detector,
            self.calib.detector.dus,
            triggers,
            filename=filename,
            label="Trigger rate [Hz]",
        )
        self.cprint("Plot up to date")

    def _update_calib(self, det_id, run):
        self.calib = kp.calib.Calibration(det_id=det_id, run=run)


def main():
    from docopt import docopt

    args = docopt(__doc__, version=VERSION)

    det_id = int(args["-d"])
    plots_path = args["-o"]
    ligier_ip = args["-l"]
    ligier_port = int(args["-p"])

    pipe = kp.Pipeline()
    pipe.attach(MemoryObserver, every=1000)
    pipe.attach(
        kp.io.ch.CHPump,
        host=ligier_ip,
        port=ligier_port,
        tags="IO_EVT",
        timeout=60 * 60 * 24 * 7,
        max_queue=2000,
    )
    pipe.attach(EventExtractor)
    pipe.attach(TriggeredHits, det_id=det_id, plots_path=plots_path)
    pipe.drain()


if __name__ == "__main__":
    main()
