#!/usr/bin/env julia

if length(ARGS) != 2
    println("Usage: ./fibonaccifit.jl LIGIER_HOST LIGIER_PORT")
    exit(1)
end

println("Initialising libraries, this may take a minute...")
using Sockets
using KM3io
using NeRCA
using CairoMakie

const LIGIER_HOST = getalladdrinfo(ARGS[1])[1]
const LIGIER_PORT = parse(Int, ARGS[2])

function main()
    println("Starting live FibonacciFit")

    detector = Detector("/data/ffit.datx")
    @show detector
    fig_indices = Dict{Int, Tuple{Int, Int}}()
    row = 1
    col = 1
    for s in detector.strings
        if col > 6
            row += 1
            col = 1
        end
        fig_indices[s] = (row, col)
        col += 1
    end

    msfparams = MuonScanfitParameters(;tmaxlocal=18.0, roadwidth=200.0, nfits=12)
    msfit = MuonScanfit(msfparams, detector)
    @show msfparams

    idx = 0
    for event in CHClient{KM3io.DAQEvent}(LIGIER_HOST, LIGIER_PORT)
        idx += 1
        muons = msfit(event.snapshot_hits)
        best_fit = first(muons)
        flag = "."
        if best_fit.Q > 0
            flag = "+"
            chits = calibrate(detector, event.snapshot_hits)
            if best_fit.dir.z > 0
                flag = "U"
                fig = Figure(size=(600,400))
                for s in detector.strings
                    col, row = fig_indices[s]
                    _chits = filter(h->h.string == s, chits)
                    ts = [h.t for h in _chits]
                    zs = [h.pos.z for h in _chits]
                    ax = Axis(fig[col, row])
                    scatter!(ax, ts, zs)
                end
                save("/plots/ztplot_ffit.png", fig)
            end
            if best_fit.dir.z < -0.99
                flag = "T"
                write_timeresiduals("/data/reco_timeres.csv", event.header.run, best_fit, chits)
            end
        end
        print(flag)
        if idx % 80 == 0
            println()
            flush(stdout)
        end
    end
end


function write_timeresiduals(fname::AbstractString, run_id, fit, calibrated_hits)
    if !isfile(fname)
        fobj = open(fname, "w")
        write(fobj, "run,timestamp,du,floor,dom_id,t_res,Q\n")
    else
        fobj = open(fname, "a")
    end
    t = Int(round(time()))
    for hit in calibrated_hits
        cphoton = cherenkov(fit, hit)
        cphoton.d_closest > 100 && continue  # skip DOMs which are too far away
        write(fobj, "$(run_id),$(t),$(hit.string),$(hit.floor),$(hit.dom_id),$(cphoton.Δt),$(fit.Q)\n")
    end
    flush(fobj)
end


main()
